#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <curl/curl.h>
#include "tags/include/tags.h"
#include "queue.h"
#include "fmt.h"

static item_t **q_queue;
static int q_nitems;

static void q_item_free(item_t *item)
{
	if (!item)
		return;
	curl_free(item->artist);
	curl_free(item->title);
	curl_free(item->utctime);
	curl_free(item->mb);
	curl_free(item->album);
	free(item);
}

void q_push(metatag_t *meta, int len)
{
	item_t *item;
	
	q_queue = realloc(q_queue, (q_nitems + 1) * sizeof(item_t *));
	
	item = malloc(sizeof(item_t));
	item->artist = fmt_escape(meta->artist);
	item->title = fmt_escape(meta->title);
	item->utctime = fmt_escape(fmt_timestr(time(NULL), 1));
	snprintf(item->len, sizeof(item->len), "%d", len);
	if(meta->mb == NULL)
		item->mb = fmt_escape("");
	else
		item->mb = fmt_escape(meta->mb);
	if(meta->album == NULL)
		item->album = fmt_escape("");
	else
		item->album = fmt_escape(meta->album);
	q_queue[q_nitems++] = item;
}

item_t *q_push2(char *artist, char *title, char *len, char *time,
		char *album, char *mb)
{
	char *temp = NULL;
	item_t *item;

	q_queue = realloc(q_queue, (q_nitems + 1) * sizeof(item_t *));

	item = calloc(1, sizeof(item_t));
	temp = fmt_unescape(artist);
	item->artist = fmt_escape(temp);
	curl_free(temp);
	temp = fmt_unescape(title);
	item->title = fmt_escape(temp);
	curl_free(temp);
	memcpy(item->len, len, sizeof(len));
	temp = fmt_unescape(time);
	item->utctime = fmt_escape(temp);
	curl_free(temp);
	temp = fmt_unescape(album);
	item->album = fmt_escape(temp);
	curl_free(temp);
	temp = fmt_unescape(mb);
	item->mb = fmt_escape(temp);
	curl_free(temp);

	q_queue[q_nitems++] = item;

	return item;
}

item_t *q_peek(void)
{
	if (!q_nitems)
		return NULL;
	return q_queue[q_nitems - 1];
}

item_t *q_peekall(int rewind)
{
	static int item;
	if (rewind) {
		item = q_nitems;
		return NULL;
	}
	if (--item >= 0)
		return q_queue[item];
	return NULL;
}

int q_pop(void)
{
	item_t *item;
	
	if (!q_nitems)
		return 0;
	item = q_queue[--q_nitems];
	if (!q_nitems) {
		free(q_queue);
		q_queue = NULL;
	} else
		q_queue = realloc(q_queue, q_nitems * sizeof(item_t *));
	q_item_free(item);
	return -1;
}

void q_free(void)
{
	while (q_pop());
}

int q_len(void)
{
	return q_nitems;
}
